@@ -1,5 +1,3 @@
-import ast
-import inspect
 import sys
 
 from astkit.render import SourceCodeRenderer as renderer
@@ -7,13 +5,8 @@ from astkit.render import SourceCodeRenderer as renderer
 from instrumental.compat import exec_f
 from instrumental.instrument import CoverageAnnotator
 from instrumental.recorder import ExecutionRecorder
-
-def load_module(func):
-    source = inspect.getsource(func)
-    normal_source =\
-        "\n".join(line[12:] for line in source.splitlines(False)[1:])
-    module = ast.parse(normal_source)
-    return module, normal_source
+from instrumental.test import DummyConfig
+from instrumental.test import load_module
 
 class TestInstrumentation(object):
     
@@ -28,16 +21,32 @@ class TestInstrumentation(object):
     
     def _load_and_compile_module(self, module_func):
         module, source = load_module(module_func)
-        self.recorder.add_source(module_func.__name__, source)
-        transformer = CoverageAnnotator(module_func.__name__,
+        from instrumental.pragmas import PragmaFinder
+        pragmas = PragmaFinder().find_pragmas(source)
+        from instrumental.metadata import MetadataGatheringVisitor
+        config = DummyConfig()
+        self.recorder.add_metadata(MetadataGatheringVisitor.analyze(config,
+                                                                    module_func.__name__,
+                                                                    source, 
+                                                                    pragmas))
+        # self.recorder.add_source(module_func.__name__, source)
+        transformer = CoverageAnnotator(config,
+                                        module_func.__name__,
                                         self.recorder)
         inst_module = transformer.visit(module)
         sys.stdout.write(renderer.render(inst_module) + "\n")
-        #for node in ast.walk(inst_module):
-        #    print node, node.__dict__
         code = compile(inst_module, '<string>', 'exec')
         return code
     
+    def _verify_conditions(self, module, label, expected):
+        construct = self.recorder.metadata[module.__name__].constructs[label]
+        tag = self.recorder.DEFAULT_TAG
+        for i, value in enumerate(expected):
+            if expected[i]:
+                assert tag in construct.conditions[i], construct.conditions[i]
+            else:
+                assert not construct.conditions[i], construct.conditions[i]
+    
     def test_two_pin_and_t_t(self):
         def test_module():
             a = True
@@ -47,12 +56,8 @@ class TestInstrumentation(object):
         exec_f(code, globals())
         assert True == result
         
-        recorder = self.recorder
-        label = 1
-        assert label in recorder._constructs
-        assert recorder._constructs[label].conditions[0]
-        assert not recorder._constructs[label].conditions[1]
-        assert not recorder._constructs[label].conditions[2]
+        self._verify_conditions(test_module, '3.2', 
+                                [True, False, False])
     
     def test_two_pin_and_t_f(self):
         def test_module():
@@ -63,12 +68,8 @@ class TestInstrumentation(object):
         exec_f(code, globals())
         assert False == result
         
-        recorder = self.recorder
-        label = 1
-        assert label in recorder._constructs
-        assert not recorder._constructs[label].conditions[0]
-        assert not recorder._constructs[label].conditions[1]
-        assert recorder._constructs[label].conditions[2]
+        self._verify_conditions(test_module, '3.2', 
+                                [False, False, True])
     
     def test_two_pin_and_f_t(self):
         def test_module():
@@ -79,12 +80,8 @@ class TestInstrumentation(object):
         exec_f(code, globals())
         assert False == result
         
-        recorder = self.recorder
-        label = 1
-        assert label in recorder._constructs
-        assert not recorder._constructs[label].conditions[0]
-        assert recorder._constructs[label].conditions[1]
-        assert not recorder._constructs[label].conditions[2]
+        self._verify_conditions(test_module, '3.2', 
+                                [False, True, False])
     
     def test_two_pin_and_f_f(self):
         def test_module():
@@ -95,12 +92,8 @@ class TestInstrumentation(object):
         exec_f(code, globals())
         assert False == result
         
-        recorder = self.recorder
-        label = 1
-        assert label in recorder._constructs
-        assert not recorder._constructs[label].conditions[0]
-        assert recorder._constructs[label].conditions[1]
-        assert not recorder._constructs[label].conditions[2]
+        self._verify_conditions(test_module, '3.2', 
+                                [False, True, False])
     
     def test_two_pin_or_t_t(self):
         def test_module():
@@ -111,12 +104,8 @@ class TestInstrumentation(object):
         exec_f(code, globals())
         assert True == result
         
-        recorder = self.recorder
-        label = 1
-        assert label in recorder._constructs
-        assert recorder._constructs[label].conditions[0]
-        assert not recorder._constructs[label].conditions[1]
-        assert not recorder._constructs[label].conditions[2]
+        self._verify_conditions(test_module, '3.2', 
+                                [True, False, False])
     
     def test_two_pin_or_t_f(self):
         def test_module():
@@ -127,12 +116,8 @@ class TestInstrumentation(object):
         exec_f(code, globals())
         assert True == result
         
-        recorder = self.recorder
-        label = 1
-        assert label in recorder._constructs
-        assert recorder._constructs[label].conditions[0]
-        assert not recorder._constructs[label].conditions[1]
-        assert not recorder._constructs[label].conditions[2]
+        self._verify_conditions(test_module, '3.2', 
+                                [True, False, False])
     
     def test_two_pin_or_f_t(self):
         def test_module():
@@ -143,12 +128,8 @@ class TestInstrumentation(object):
         exec_f(code, globals())
         assert True == result
         
-        recorder = self.recorder
-        label = 1
-        assert label in recorder._constructs
-        assert not recorder._constructs[label].conditions[0]
-        assert recorder._constructs[label].conditions[1]
-        assert not recorder._constructs[label].conditions[2]
+        self._verify_conditions(test_module, '3.2', 
+                                [False, True, False])
     
     def test_two_pin_or_f_f(self):
         def test_module():
@@ -159,12 +140,8 @@ class TestInstrumentation(object):
         exec_f(code, globals())
         assert False == result
         
-        recorder = self.recorder
-        label = 1
-        assert label in recorder._constructs
-        assert not recorder._constructs[label].conditions[0]
-        assert not recorder._constructs[label].conditions[1]
-        assert recorder._constructs[label].conditions[2]
+        self._verify_conditions(test_module, '3.2', 
+                                [False, False, True])
     
     def test_three_pin_and_t_t_t(self):
         def test_module():
@@ -176,13 +153,8 @@ class TestInstrumentation(object):
         exec_f(code, globals())
         assert True == result
         
-        recorder = self.recorder
-        label = 1
-        assert label in recorder._constructs
-        assert recorder._constructs[label].conditions[0]
-        assert not recorder._constructs[label].conditions[1]
-        assert not recorder._constructs[label].conditions[2]
-        assert not recorder._constructs[label].conditions[3]
+        self._verify_conditions(test_module, '4.2', 
+                                [True, False, False, False])
     
     def test_three_pin_and_f_t_t(self):
         def test_module():
@@ -194,13 +166,8 @@ class TestInstrumentation(object):
         exec_f(code, globals())
         assert False == result
         
-        recorder = self.recorder
-        label = 1
-        assert label in recorder._constructs
-        assert not recorder._constructs[label].conditions[0]
-        assert recorder._constructs[label].conditions[1]
-        assert not recorder._constructs[label].conditions[2]
-        assert not recorder._constructs[label].conditions[3]
+        self._verify_conditions(test_module, '4.2', 
+                                [False, True, False, False])
     
     def test_three_pin_and_t_f_t(self):
         def test_module():
@@ -212,13 +179,8 @@ class TestInstrumentation(object):
         exec_f(code, globals())
         assert False == result
         
-        recorder = self.recorder
-        label = 1
-        assert label in recorder._constructs
-        assert not recorder._constructs[label].conditions[0]
-        assert not recorder._constructs[label].conditions[1]
-        assert recorder._constructs[label].conditions[2]
-        assert not recorder._constructs[label].conditions[3]
+        self._verify_conditions(test_module, '4.2', 
+                                [False, False, True, False])
     
     def test_three_pin_and_t_t_f(self):
         def test_module():
@@ -230,13 +192,8 @@ class TestInstrumentation(object):
         exec_f(code, globals())
         assert False == result
         
-        recorder = self.recorder
-        label = 1
-        assert label in recorder._constructs
-        assert not recorder._constructs[label].conditions[0]
-        assert not recorder._constructs[label].conditions[1]
-        assert not recorder._constructs[label].conditions[2]
-        assert recorder._constructs[label].conditions[3]
+        self._verify_conditions(test_module, '4.2', 
+                                [False, False, False, True])
     
     def test_three_pin_and_f_f_f(self):
         def test_module():
@@ -248,13 +205,8 @@ class TestInstrumentation(object):
         exec_f(code, globals())
         assert False == result
         
-        recorder = self.recorder
-        label = 1
-        assert label in recorder._constructs
-        assert not recorder._constructs[label].conditions[0]
-        assert recorder._constructs[label].conditions[1]
-        assert not recorder._constructs[label].conditions[2]
-        assert not recorder._constructs[label].conditions[3]
+        self._verify_conditions(test_module, '4.2', 
+                                [False, True, False, False])
     
     def test_instrument_if_true_result(self):
         def test_module():
@@ -267,11 +219,7 @@ class TestInstrumentation(object):
         exec_f(code, globals())
         assert result is 1
         
-        recorder = self.recorder
-        label = 1
-        assert label in recorder._constructs
-        assert recorder._constructs[label].conditions[True]
-        assert not recorder._constructs[label].conditions[False]
+        self._verify_conditions(test_module, '2.1', [False, True])
         
     def test_instrument_if_false_result(self):
         def test_module():
@@ -284,11 +232,7 @@ class TestInstrumentation(object):
         exec_f(code, globals())
         assert result is 7
         
-        recorder = self.recorder
-        label = 1
-        assert label in recorder._constructs
-        assert not recorder._constructs[label].conditions[True]
-        assert recorder._constructs[label].conditions[False]
+        self._verify_conditions(test_module, '2.1', [True, False])
         
     def test_instrument_while_false_result(self):
         def test_module():
@@ -301,11 +245,7 @@ class TestInstrumentation(object):
         exec_f(code, globals())
         assert result is 0
         
-        recorder = self.recorder
-        label = 1
-        assert label in recorder._constructs
-        assert not recorder._constructs[label].conditions[True]
-        assert recorder._constructs[label].conditions[False]
+        self._verify_conditions(test_module, '3.1', [True, False])
         
     def test_instrument_while_with_loop_result(self):
         def test_module():
@@ -318,12 +258,8 @@ class TestInstrumentation(object):
         exec_f(code, globals())
         assert result is 3
         
-        recorder = self.recorder
-        label = 1
-        assert label in recorder._constructs
-        assert recorder._constructs[label].conditions[True]
-        assert recorder._constructs[label].conditions[False]
-    
+        self._verify_conditions(test_module, '3.1', [True, True])
+            
     def test_instrument_ifexp_true(self):
         def test_module():
             a = 3
@@ -332,11 +268,7 @@ class TestInstrumentation(object):
         exec_f(code, globals())
         assert result is 1
         
-        recorder = self.recorder
-        label = 1
-        assert label in recorder._constructs
-        assert recorder._constructs[label].conditions[True]
-        assert not recorder._constructs[label].conditions[False]
+        self._verify_conditions(test_module, '2.1', [False, True])
     
     def test_instrument_ifexp_false(self):
         def test_module():
@@ -346,12 +278,8 @@ class TestInstrumentation(object):
         exec_f(code, globals())
         assert result is 4
         
-        recorder = self.recorder
-        label = 1
-        assert label in recorder._constructs
-        assert not recorder._constructs[label].conditions[True]
-        assert recorder._constructs[label].conditions[False]
-
+        self._verify_conditions(test_module, '2.1', [True, False])
+    
     def test_instrument_ifexp_true_and_false(self):
         def test_module():
             def test_func(arg):
@@ -361,11 +289,7 @@ class TestInstrumentation(object):
         code = self._load_and_compile_module(test_module)
         exec_f(code, globals())
         
-        recorder = self.recorder
-        label = 1
-        assert label in recorder._constructs
-        assert recorder._constructs[label].conditions[True]
-        assert recorder._constructs[label].conditions[False]
+        self._verify_conditions(test_module, '2.1', [True, True])
     
     def test_instrument_ifexp_with_boolop(self):
         def test_module():
@@ -376,9 +300,4 @@ class TestInstrumentation(object):
         code = self._load_and_compile_module(test_module)
         exec_f(code, globals())
         
-        recorder = self.recorder
-        label = 1
-        assert label in recorder._constructs
-        assert not recorder._constructs[label].conditions[0] # TT
-        assert not recorder._constructs[label].conditions[1] # FT
-        assert recorder._constructs[label].conditions[2]     # TF
+        self._verify_conditions(test_module, '2.2', [False, False, True])
